#  parse_outfile.R - Extract results from PRESENCE output file

##**********************  some utility functions needed by  parse_outfile #######

getbetas <- function(vv,x.dm) {
  if (length(vv)<1) return(NULL) else {
    vals=as.numeric(unlist(strsplit(gsub(" +",",",gsub(".+: +","",vv)),",")))
    vals=matrix(vals,ncol=2,byrow=T);
    rn=gsub('^.[0-9]+ +','',gsub(' +:.+','',vv)); cols=length(rn)
    for (i in 1:cols) {
      j=which.max(x.dm[,i]!="0")
      if (x.dm[j,i]=="1") rn[i]=paste0(rownames(x.dm)[j],".Int") else rn[i]=paste0(rownames(x.dm)[j],'.',x.dm[j,i])
    }
    dup=which(duplicated(rn));
    if (length(dup)>0) rn[dup]=paste0(rn[dup],round(runif(length(dup)),4))
    return(data.frame(est=vals[,1],se=vals[,2],row.names=rn[1:nrow(vals)]))
  }
}
check_conv_warn<-function(v,quiet=1){
  warn.conv=7; i<-grep("Completion-code",v);
  if (length(i)>0) {
    cc=gsub('.+: ','',v[i])
    if(cc=="130") {
      warn.conv<-as.numeric(gsub('.+: ','',v[i+2]))
      if (quiet==0) cat("\nWarning: Numerical convergence may not have been reached. ",
                        "Parameter estimates converged to approximately ", warn.conv," signifcant digits.\n\n")
    }
    if(cc=="131") {
      if (quiet==0) cat("\nError: Numerical convergence aborted ** Iterations > MaxFN ** Increase Maxfn.\n\n")
    }
  }
  return(warn.conv)
}
check_VC_warn<-function(v){
  warn.VC=NULL; instns<-grep("neg. std.err",v);  l<-length(instns)
  if(l>0) warn.VC<-TRUE
  return(warn.VC)
}
getreal <- function(v1,i) {
  v2=unlist(strsplit(gsub(' +',' ',gsub('.+: +','',v1[i])),' '))
  R.est=matrix(v2,ncol=5,byrow=T)[,-4]
  R.est=apply(R.est,2,as.numeric)
  rownames(R.est)=gsub(' +','_',gsub(' +: .+','',v1[i]))
  colnames(R.est)=c('est','se','ciLO','ciHI')
  return(R.est)
}
####################################################################

parse_outfile <- function(outfile) {

   ###############################################################
   #### Extract results from output file
   ################################################################

  a=readLines(outfile)
i=grep('Number of sites',a); ns=as.numeric(gsub('.+=','',a[i]))
i=grep('Number of surveys',a); nsrvys=as.numeric(gsub('.+=','',a[i]))
i=grep('==>name=',a); modname=gsub('.+=','',a[i])

dmi=grep("^Matrix ",a); dm=list()
for (i in 1:length(dmi)) {
  dm[[i]]=NULL
  rows=as.numeric(gsub(',.+','',gsub('.+rows=','',a[dmi[i]])))
  if (rows>0) {
    cols=as.numeric(gsub('.+cols=','',a[dmi[i]]))
    a1=a[dmi[i]+1+1:rows]
     a2=unlist(strsplit(gsub(' +',' ',a1),' '))
     a3=matrix(a2,ncol=cols,byrow=T)
     dm[[i]]=a3[,-1]; rownames(dm[[i]])=a3[,1]
  }
}

psi.dm=dm[[1]]; p.dm=dm[[5]]
npar=(ncol(psi.dm)+ncol(p.dm))
v=a; l=length(v);

### get PRESENCE version
i=grep("Version",v,fixed=TRUE)[1]; version=gsub(".+Version ","",v[i])

## find AIC value for each model
i=grep("-2log.likelihood",v); neg2loglike=as.numeric(gsub(".+= ","",v[i])); aic=as.numeric(gsub(".+= ","",v[i+1]))
v=v[-1:-i]

### get Untransformed coefficients (Beta's)
i=grep("^A.+ : ",v); psi.coeff=getbetas(v[i],psi.dm)
j=grep('[C]*R',rownames(psi.coeff)); R.coeff=psi.coeff[j,]; psi.coeff=psi.coeff[-j,];
i=grep("^A.+[C]*psi.+ :",v); j=grep("^A.+[C]*R.+ :",v)
k=grep("^[B-F][0-9].+ : ",v);
p.coeff=getbetas(v[k],p.dm)
l=grep("^delta",rownames(p.coeff)); delta.coeff=p.coeff[l,]; p.coeff=p.coeff[-l,]
k=grep("^[B-F].+ p.+ :",v); l=grep("^[B-F].+ delta.+ :",v)

### get_VC matrix
ii=grep("^[A-F][0-9].+ : ",v); names=gsub(" .+","",v[ii]); npar=length(names)
i2=grep("Variance-Covariance Matrix of",v,fixed=TRUE)
if (length(i2)>0) {
  v1=gsub('^ +[A-F][0-9]+ +','',gsub(' +',' ',v[i2+1+1:npar]))
  v2=as.numeric(unlist(strsplit(v1,' ')))
  VC=matrix(v2,ncol=npar,byrow=T); rownames(VC)=colnames(VC)=names

  psi.VC=as.matrix(VC[1:length(i), 1:length(i)])
  R.VC  =as.matrix(VC[length(i)+1:length(j), length(i)+1:length(j)])
  p.VC  =as.matrix(VC[length(c(i,j))+1:length(k), length(c(i,j))+1:length(k)])
  delta.VC  =as.matrix(VC[length(c(i,j,k))+1:length(l), length(c(i,j,k))+1:length(l)])
} else psi.VC=R.VC=p.VC=delta.VC=NULL

psi.est=p.est=psi_c.est=R.est=delta.est=NULL

##### check for warnings
warn.conv=check_conv_warn(v)
warn.VC=check_VC_warn(v)

i=grep('Real parameter estimates',v)
if (length(i)>0) {
  # read real parameter estimates
  v1=v[-1:-(i+3)];
  i=c(grep('^psi',v1),grep('^Cpsi',v1)); psi.est=getreal(v1,i)
  i=c(grep('^R',v1),grep('^CR',v1)); R.est=getreal(v1,i)
  i=grep('^delta',v1); delta.est=getreal(v1,i)
  i=grep('^p',v1); p.est=getreal(v1,i)
}


result=list(modname=modname,
            model=list(psi=NULL,R=NULL,Cpsi=NULL,CR=NULL,delta=NULL,p=NULL),
            dmat=list(psi=psi.dm,p=p.dm),
            data=NULL,outfile=outfile,
            neg2loglike=neg2loglike,npar=npar, aic=aic,
            beta=list(psi=  as.data.frame(psi.coeff),  psi.VC=psi.VC,
                      R  =  as.data.frame(R.coeff),    R.VC=  R.VC,
                      delta=as.data.frame(delta.coeff),delta.VC=delta.VC,
                      p  =  as.data.frame(p.coeff),    p.VC=p.VC,VC=VC),
            real=list(psi=as.data.frame(psi.est),
                      R=as.data.frame(R.est),
                      delta=as.data.frame(delta.est),
                      p=as.data.frame(p.est),
                      warnings=list(conv=warn.conv,VC=warn.VC)))

class(result)=c("occMod","doMs2")
return(result)
}
