% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genpres.R
\name{LRTpower}
\alias{LRTpower}
\title{compute expected power using non-central chi-square function}
\usage{
LRTpower(ch, df = 1, alf = 0.05)
}
\arguments{
\item{ch}{chi-square value from likelihood-ratio test}

\item{df}{number of degrees of freedom}

\item{alf}{significance level (default=0.05)}
}
\value{
power = prob. to reject Ho
}
\description{
\code{LRTpower} compute expected power using non-central chi-square function
where power is defined as the probability of rejecting the null hypothesis
when it is false.
}
\details{
A description of an analytical-numerical approximation
approach can be found in:

\emph{Burnham, K.P., D.R. Anderson, G.C. White, C. Brownie and K.H. Pollock. 1987.
Design and Analysis Methods for Fish Survival Experiments Based on
Release-Recapture. Am. Fish. Soc. Monogr. 5, pages 216, 289-295.}

The basic idea: Given the alternative model, Ha, is true, what is the
probability that the likelihood-ratio test will reject the null model, Ho? If
Ho is true, the power of the test should equal the significance level alfa.

Data should be generated by computing expected values under the alternative
model for specified sample sizes, compute estimates under the alternative and
null models, then compute a likelihood-ratio test between models. The
likelihood-ratio test will give you the chi-square value and degrees of freedom needed
for the power analysis.

Specifically, the chi-square value approximates the noncentrality parameter of a
noncentral chi-square distribution, from which power is directly obtained.
}
\examples{
\dontrun{
   # compute power of likelihood-ratio test using default alfa=0.05
# power=LRTpower(chisq1-chisq2,npar1-npar2)
}
}
