% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_skinks.R
\name{exampleSkinks}
\alias{exampleSkinks}
\title{Example intro script for RPresence using Skinks data from book.}
\usage{
exampleSkinks()
}
\description{
Dummy function for help display
}
\examples{
#  --- test multi-season model using "Skinks" data ----

#    Read detection data from csv file: (and remove 1st column(sitenames))
skdata=read.csv(system.file('extdata/skinks.csv',package="RPresence"),na.strings='-')[,-1]
detdata=skdata[,1:15]

#    Get site covariates - Pasture, Tussock
Pasture = skdata[,16]
#    No need to worry about Tussock as it is simply the converse of Pasture (1-Pasture)

#    Create Presence input object using "createPao".  Since this is a multi-season dataset,
#    we need to specify the number of surveys per season.  In this case there were 5 seasons,
#    with 3 surveys each season.
data=createPao(detdata, unitcov=data.frame(Pasture), nsurveyseason=rep(3,5))

#    Run model with all parameters constant.  Multi-season models have 2 new parameters,
#    gamma(colonization) and epsilon(extinction) which apply to the intervals between
#    seasons.  They can be functions of site covariates, or as functions of "seasonal"
#    covariates, which can be created as a data-frame and passed to the occMod function.

m0<-occMod(model=list(psi~1,gamma~1,epsilon~1,p~1),data=data,type="do.1");

#    Run model with initial occupancy as a function of the habitat covariate.  Since there are
#    only 2 possible values for the habitat covariate: "Pasture" or "Tussock", we can use
#    either one in the formula.  The only difference will be in the beta estimates.  The
#    intercept beta will be occupancy for Pasture here, and the effect beta will be the
#    difference between occupancy for pasture and tussock.  Real estimates of psi will be the
#    same as if we used tussock instead of pasture in the formula for psi.

m1<-occMod(model=list(psi~Pasture,gamma~1,epsilon~1,p~1),data=data,type="do.1");

#    Run model with init occupancy as above, and colonization different each season.  To
#    emphasize the point above, we are using the tussock covariate here.  The intercept beta
#    in this model will be occupancy for tussock, and the effect beta will be the difference
#    between tussock and pasture.  Real estimates of psi will be the same as if we used
#    pasture instead of tussock in the formula for psi.
m2<-occMod(model=list(psi~Pasture,gamma~SEASON,epsilon~1,p~1),
                        data=data,type="do.1",outfile='modname');

cat('skinks AIC:',m0$aic,'\n');
aictable=createAicTable(list(m0,m1,m2))
print(summary(aictable))

}
\seealso{
\code{\link{exampleGenpres}}, \code{\link{exampleBobcats}}, \code{\link{exampleSalamanders}}
}
