% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genpres.R
\name{genpresRPC}
\alias{genpresRPC}
\title{generate simulated data detection histories Royle point-count model}
\usage{
genpresRPC(N = 100, K = 4, lambda = 2, r = 0.2, seed = 0)
}
\arguments{
\item{N}{number of sites}

\item{K}{number of surveys}

\item{lambda}{avg. number of individuals per site}

\item{r}{probability of detection of an individual, given occupied state}

\item{seed}{seed number for random number generation}
}
\value{
list(hst=det. history matrix, frq=vector of frequencies)
}
\description{
\code{genpresEV2sp} generates expected value detection histories for
Royle point-count model
}
\note{
This function generates histories with counts, instead of
         just 0's and 1's.
}
\examples{
\dontrun{
# generate expected value data for 2-species single-season model
x1=genpresSD2sp(N=100,K=4,
            psi=c(.8,.3,.7),            # psiA, psiBA, psiBa
            p=c(.6,.66,.7,.4,.5),       #  pA, pB, rA, rBA, rBa
            gam=c(0, 0, 0),             #  gamA, gamBA, gamBa
            eps=c(0, 0, 0),             #  epsA, epsBA, epsBa
            SPS=4)                      #  surveys per season

# generate expected value data for 2-species multi-season model
x2=genpresSD2sp(N=100,K=4,              # 100 sites, 4 surveys
            psi=c(.8,.3,.7),            # psiA, psiBA, psiBa
            p=c(.6,.66,.7,.4,.5),       #  pA, pB, rA, rBA, rBa
            gam=c(.1, .14, .17),        #  gamA, gamBA, gamBa
            eps=c(.2, .22, .24),        #  epsA, epsBA, epsBa
            SPS=2)                      #  surveys per season
}
}
\seealso{
\code{\link{genpresEV}},\code{\link{genpresSD}},
         \code{\link{genpresSDms}},  \code{\link{genpresEVms}}
}
