% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genpres.R
\name{genpresSDcd}
\alias{genpresSDcd}
\title{generate stochastic data detection histories for corr. det. model}
\usage{
genpresSDcd(
  N = 100,
  K = 3,
  psi = 0.75,
  p = 0.4,
  theta = c(1, 1),
  th0pi = 0,
  gamma = 0,
  epsilon = 0,
  SPS = NA,
  seed = 0
)
}
\arguments{
\item{N}{number of sites}

\item{K}{number of surveys (segments for spatial replication)}

\item{psi}{probability of occupancy}

\item{p}{probability of detection (not false positive)}

\item{theta}{probabilities of local occupancy (th0, th1)}

\item{th0pi}{probability of local occupancy before 1st survey}

\item{gamma}{probability of colonization}

\item{epsilon}{probability of extinction}

\item{SPS}{number of surveys per season}

\item{seed}{seed number for random number generation}
}
\value{
list(hst=det. history matrix, frq=vector of frequencies)
}
\description{
\code{genpresEVcd} generates stochastic data  detection history frequencies for
single or multi-season correlated detection models
}
\note{
This function generates at least 2^K detection-histories, so if K>8, it may take a
      very long time to run.
}
\examples{
\dontrun{
# generate expected value data for standard single-season model (no corr.det.)
hstfrq=genpresEVcd(N=1000, K=4, psi=.8, p=0.3, theta=c(1,1), th0pi=0)
EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file

   # generate expected value data for single-season model with corr.det.
hstfrq=genpresEVcd(N=1000, K=4, psi=.8, p=.3, theta=c(.2, .7), th0pi=.4)
EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file

rm(list=ls()); N=10000; K=4; psi=.75; p=.4; theta=c(.3, .6); th0pi=0; gamma=0; epsilon=0; SPS=K
}
}
\seealso{
\code{\link{genpresEVms}},\code{\link{genpresSD}},
         \code{\link{genpresSDms}}, \code{\link{genpresEV}}
}
