% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genpres.R
\name{genpresSDmm}
\alias{genpresSDmm}
\title{generate simulated single-season, multi-method data detection histories}
\usage{
genpresSDmm(
  N = 100,
  K = 4,
  psi = 0.75,
  theta = 0.4,
  p = 0.4,
  methods = 2,
  gam = NA,
  eps = NA,
  SPS = 8888,
  seed = 0
)
}
\arguments{
\item{N}{number of sites}

\item{K}{number of surveys}

\item{psi}{probability of occupancy OR a pair of beta values}

\item{theta}{probability of local occupancy  or vector}

\item{p}{probability of detection scaler or vector}

\item{methods}{number of methods per survey}

\item{gam}{probability of local colonization}

\item{eps}{probability of local extinction}

\item{SPS}{number of surveys per season}

\item{seed}{seed number for random number generation}
}
\value{
list(hst=det. history matrix, frq=vector of frequencies)
}
\description{
\code{genpresSDmm} generates stochastic detection history frequencies for
single-season, multi-method models
}
\examples{
\dontrun{
   # generate simulated value data for standard single-season model
   # with 500 sites, 2 methods for each of 4 surveys, psi=0.8,
   # local occupancy = .2, .3, .4, .5 for surveys 1,2,3 and 4,
   # detection probs = .3 for 1st method, .8 for 2nd method

simdata=genpresSDmm(N=500,                   # 500 sites
            K=8,                             # 4 surveys * 2 methods
            psi=0.8,                         # occupancy prob.
            theta=c(.2,.3,.4,.5),            # local occ. prob each survey
            p=c(.3,.8, .3,.8, .3,.8, .3,.8), # det.prob each survey/method
            methods=2)                         # 2 methods per survey

SDpao=createPao(simdata$hst, frq=simdata$frq, methods=2)    #  create pao file

#  run single-season, multi-method (so.mm) model
m1=occMod(model=list(psi~1, p~METHOD, theta~SURVEY),data=pao, type="so.mm")
}
}
\seealso{
\code{\link{genpresSDms}},\code{\link{genpresEV}},
         \code{\link{genpresEVms}}
}
