% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/occ.mod.rpc.R
\name{occMod_RPC}
\alias{occMod_RPC}
\title{Fit static occupancy/single season model}
\usage{
occMod_RPC(
  lambda = call(),
  lambda.cov = data$unitcov,
  r = call(),
  r.cov = cbind(data$unitcov, data$survcov),
  nic = FALSE,
  modname = NULL,
  outfile,
  model = 100,
  fixed = NULL,
  initvals = NULL,
  data,
  conf,
  miscopts
)
}
\arguments{
\item{lambda}{the right-hand side of the formula for the model to fit for average individuals/site}

\item{lambda.cov}{a data frame containing the unit-specific covariates to use for lambda
component of the model.}

\item{r}{the right-hand side of the formula for the model to fit for detection probability.}

\item{r.cov}{a data frame containing the survey-specific covariates to use for the detection
component of the model.}

\item{nic}{option to select Royle-Nichols heterogenity model instead of Royle point-count model}

\item{modname}{(optional) a string containing the model name}

\item{outfile}{name for output file (use outfile='modname') for outfile named via model name}

\item{model}{the PRESENCE model code. DO NOT CHANGE.}

\item{fixed}{a single-column matrix containing values for real parameters to be fixed at.
\code{rownnames(fixed)} should contain the index of the real parameters to be fixed.}

\item{initvals}{initial values for the beta parameters at which PRESENCE begins the optimisation.
The default values in PRESENCE is 0.}

\item{data}{the \code{pao} data object containing the detection data and other information.}

\item{conf}{level for confidence interval (may be vector valued).}

\item{miscopts}{miscelleneous options (eg., maxfn=32000)}
}
\value{
returns a list of class \code{"occMod"} and \code{"rpc"}.

\code{occMod$beta} contains the objects:
 #'\item{lambda}{estimated logistic regression coefficients and standard errors for avg. individuals/site.}
\item{lambda.VC}{variance-covariance matrix for logistic regression coefficients for lambda.}
\item{r}{estimated logistic regression coefficients and standard errors for probability of detection.}
\item{r.VC}{variance-covariance matrix for logistic regression coefficients for probability of detection.}
\item{VC}{the full variance-covariance matrix for all logistic regression coefficients.}

\code{occMod$real} contains the objects:
\item{lambda}{estimated avg. individuals/site for each sampling unit, along with standard
errors and limits of 95\% confidence interval.}
\item{r}{estimated individual probabilities of detection for each survey, along with standard errors and
limits of 95\% confidence interval.}

\code{occMod$derived} contains the objects:
\item{lambda_c}{estimated probabilities of occurrence given the detection history for each sampling
unit, along with standard errors and limits of 95\% confidence interval. Will be \code{=1} for
any unit where the species was detected at least once.}
}
\description{
This is not intended for direct use, but instead the \code{\link{occMod}} function should be used with \code{type="so"}.
}
\details{
Pre-defined covariates:
\itemize{
\item{SURVEY} {categorical covariate indicating survey (for detection parameters)}
}
}
\examples{

N=500      # sites
K=6        # surveys
lambda=2  # number of individuals per site
p=.2       # prob(detect inidividual per site,survey)

pop=rpois(N,lambda) # simulate number of individuals per site

count=NULL          # for each survey ...
for (k in 1:K) {    #    simulate number of individuals detected
  count=cbind(count, rbinom(N, pop, p))
}

pao=createPao(count)

m2=occMod(list(lambda~1, r~1), data=pao, type='rpc', outfile='modname')
print(print_one_site_estimates(m2))
print(print_one_site_estimates(m2,derived=T))

}
\seealso{
\code{\link{occMod}}, \code{pao}
}
\author{
Darryl MacKenzie and Jim Hines
}
