% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/occ.mod.so.het.R
\name{occMod_SO_het}
\alias{occMod_SO_het}
\title{Fit static occupancy/single season model}
\usage{
occMod_SO_het(
  psi = call(),
  psi.cov = data$unitcov,
  p = call(),
  p.cov = cbind(data$unitcov, data$survcov),
  pi = call(),
  modname = NULL,
  outfile,
  model = 100,
  fixed = NULL,
  initvals = NULL,
  data,
  conf,
  miscopts
)
}
\arguments{
\item{psi}{the right-hand side of the formula for the model to fit for occupancy probability.}

\item{psi.cov}{a data frame containing the unit-specific covariates to use for the occupancy
component of the model.}

\item{p}{the right-hand side of the formula for the model to fit for detection probability.}

\item{p.cov}{a data frame containing the survey-specific covariates to use for the detection
component of the model.}

\item{pi}{hetergeneity pi parameter (prop sites in group 1).}

\item{modname}{(optional) a string containing the model name}

\item{outfile}{name for output file (use outfile='modname') for outfile named via model name}

\item{model}{the PRESENCE model code. DO NOT CHANGE.}

\item{fixed}{a single-column matrix containing values for real parameters to be fixed at.
\code{rownnames(fixed)} should contain the index of the real parameters to be fixed.}

\item{initvals}{initial values for the beta parameters at which PRESENCE begins the optimisation.
The default values in PRESENCE is 0.}

\item{data}{the \code{pao} data object containing the detection data and other information.}

\item{conf}{level for confidence interval (may be vector valued).}

\item{miscopts}{char vector of options to send to Presence (eg., maxfn=32000)- last element must be ''.  #'}
}
\value{
returns a list of class \code{"occMod"} and \code{"so"}.

\code{occMod$beta} contains the objects:
 #'\item{psi}{estimated logistic regression coefficients and standard errors for probability of occurrence.}
\item{psi.VC}{variance-covariance matrix for logistic regression coefficients for probability of occurrence.}
\item{p}{estimated logistic regression coefficients and standard errors for probability of detection.}
\item{p.VC}{variance-covariance matrix for logistic regression coefficients for probability of detection.}
\item{VC}{the full variance-covariance matrix for all logistic regression coefficients.}

\code{occMod$real} contains the objects:
\item{psi}{estimated probabilities of occurrence for each sampling unit, along with standard
errors and limits of 95\% confidence interval.}
\item{p}{estimated probabilities of detection for each survey, along with standard errors and
limits of 95\% confidence interval.}

\code{occMod$derived} contains the objects:
\item{psi_c}{estimated probabilities of occurrence given the detection history for each sampling
unit, along with standard errors and limits of 95\% confidence interval. Will be \code{=1} for
any unit where the species was detected at least once.}
}
\description{
This is not intended for direct use, but instead the \code{\link{occMod}} function should be used with \code{type="so"}.
}
\examples{

# load a csv data file
filename<-system.file("extdata/bobcat.csv",package="RPresence")
csv<-read.csv(filename,as.is=T,header=F)
##
nsites=nrow(csv); nsrvys=ncol(csv)  #  set number of sites,surveys from det. data
dethist=matrix(as.integer(unlist(csv)),nrow=nsites)
##          create input "pao" object, for use with occMod function
data=createPao(dethist, title="single-sp, detection heterogeneity example")

mod1<-occMod(model=list(psi~1,p~1,pi~1),data=data,type="so.het")

}
\seealso{
\code{\link{occMod}}, \code{pao}
}
\author{
Darryl MacKenzie
}
