#include "SingAnal.h"
//****  Likelihood for joint modelling of 2 species from presence/absence data  ****//
#include <math.h>
#include <stdio.h>

double TwoSpeciesLink(double *Params, int NPar) {
    double getParam(int param_num, int param_grp, int row, int site, int srvy, double *Params, int transf, int LinkFn);
    double invlogit(double x);
    int survey, site, seenA, seenB, N=SingSpec.N, T=SingSpec.T;
    double sum = 0.0, tmp=0, pA, pB, rA=0, rB=0, rAB=0, rBA=0, rBa=0,
	             psiA, psiB=0, psiAB=0, likeA, likeB, likeAB, psiBA=0, psiBa=0;
    double lower, upper, phi, delta, alf, gam, xnu, denom, rho,
	   alfaA, alfaB, alfaAB, K, psiAb, psiaB, logitPsiAb, logitPsiBa, logtau,
       logitrAb,logitrBa, rAb, raB;
    // psi[AB] = psi[A]*psi[B]*phi; phi is parameter being estimated
    // alt parameterization: Psi[2] = psi[Ba] = Pr(occ by B|not occ by A)
    for (site=0; site<N; site++) {  // loop through individual sites
        // calculate psi[A] and psi[B]
        psiA=getParam(0,0,0,site,0,Params,0,SingSpec.LinkFn); SingSpec.realParmEst[0][site]=psiA;
		
        switch(SingSpec.Alt_Param_Checked) {
          
		  case 0:  // default parameterization - psiA, psiB, phi
            psiB=getParam(1,0,1,site,0,Params,0,SingSpec.LinkFn); 
			psiAB = psiA*psiB;
            phi=getParam(2,0,2,site,0,Params,1,expLnk);
            // check whether value is in allowable bounds
            lower = (psiA+psiB-1.0)/(psiA*psiB); if (lower<0.0) lower=0.0;
            upper = 1.0/psiA; if (upper>(1.0/psiB)) upper=1.0/psiB;
            if(phi < lower) return((fabs(phi-lower)+1.0)*1.0e10);
            if(phi > upper) return((fabs(phi-upper)+1.0)*1.0e10);
            psiAB = psiA*psiB*phi;
			SingSpec.realParmEst[1][site]=psiB;
			SingSpec.realParmEst[2][site]=phi;			
			break;
          
		  case 1:        //  new parmeterization - psiA, psiBA, psiBa
            psiBA=getParam(1,0,1,site,0,Params,0,SingSpec.LinkFn);
            psiBa=getParam(2,0,2,site,0,Params,0,SingSpec.LinkFn); 
            psiAB=psiA*psiBA;                //  Pr(occ by B) = Pr(occ by A)*pr(Occ by B | A present)
            psiB=psiA*psiBA+(1-psiA)*psiBa;  //    + Pr(not occ by A)*Pr(occ by B | A not present)
			SingSpec.realParmEst[1][site]=psiBA;
			SingSpec.realParmEst[2][site]=psiBa;
			break;
        
          case 2:    //  newer parameterzation - psiA, psiBa, odds-ratio(psiBa:psiBA)
            psiBa=getParam(1,0,1,site,0,Params,0,SingSpec.LinkFn);
            alf=50; if (psiBa<1) alf=log(psiBa/(1-psiBa));
            //alf=getParam(1,0,1,site,0,Params,0,IDLnk);    //  logit(psiBa) = alpha, 
            xnu=getParam(2,0,2,site,0,Params,0,expLnk);    //  logit(psiBA) = alpha + gamma 
            gam=-50; if (xnu>0) gam=log(xnu);
            psiBA=invlogit(alf+gam); 
			psiB=psiA*psiBA+(1-psiA)*psiBa; 
			psiAB=psiA*psiBA;  
			SingSpec.realParmEst[1][site]=psiBa;
			SingSpec.realParmEst[2][site]=xnu;			
			break;
        
          case 3:  //  newer parameterzation - psiAb, psiBa, odds-ratio(psiAb:psiA|B)
            logitPsiAb=getParam(0,0,0,site,0,Params,0,IDLnk);  //  untransformed = logit(psiAb)
            logitPsiBa=getParam(1,0,1,site,0,Params,0,IDLnk);  //  untransformed = logit(psiBa)
            logtau=getParam(2,0,2,site,0,Params,0,IDLnk);      
			tmp=exp(logitPsiAb+logitPsiBa+logtau); 
			denom=1+tmp+exp(logitPsiAb)+exp(logitPsiBa);
			psiAB=tmp/denom; 
			psiA=psiAB/invlogit(logitPsiBa+logtau); 
            psiB=psiAB/invlogit(logitPsiAb+logtau); 
			SingSpec.realParmEst[0][site]=1/exp(-logitPsiAb);
			SingSpec.realParmEst[1][site]=1/exp(-logitPsiBa);
			SingSpec.realParmEst[2][site]=exp(logtau);
			break;
        
		  case 4:  //  newest parameterzation - alpha^a,alpha^B,alpha^AB
            // these are conditional probs... psiAb = psi^(A|b)
			alfaA=getParam(0,0,0,site,0,Params,0,IDLnk);  //  logit(psiAb) = alpha^A
            alfaB=getParam(1,0,1,site,0,Params,0,IDLnk);  //  logit(psiBa) = alpha^B
            alfaAB=getParam(2,0,2,site,0,Params,0,IDLnk);  //  logit(psiAB)= alpha^A+alpha^AB 
			psiAb=exp(alfaA); 
			psiaB=exp(alfaB); 
			psiAB=exp(alfaA+alfaB+alfaAB);
			K=1+psiAb+psiaB+psiAB; psiAb/=K; psiaB/=K; psiAB/=K;
			psiA=psiAb+psiAB; psiB=psiaB+psiAB;
			SingSpec.realParmEst[0][site]=psiAb;
			SingSpec.realParmEst[1][site]=psiaB;
			SingSpec.realParmEst[2][site]=psiAB;
        }
        likeA = psiA-psiAB; likeB = psiB-psiAB; likeAB = psiAB;  seenA=seenB=0; 
        
        // P[0]=pA  pB=pB  P[2]=pAB  P[3]=pAB'  P[4]=pA'B
        // new definition of detection probabilities
        // P[2] = rA = Pr(detect A | A & B present)
        // P[3] = rB = Pr(detect B | A & B present)
        // P[4] = rAB = Pr(detect A&B | A&B present) = pA x pB if independent
        
        //calculate pA's and pB's
        for (survey=0; survey<T; survey++) {
            if (SingSpec.Data[site][survey]!=-1) {
                pA=rA=getParam(3+survey,1,survey,site,survey,Params,0,SingSpec.LinkFn);
                pB=rB=getParam(3+survey+T,1,survey+T,site,survey,Params,0,SingSpec.LinkFn);
				SingSpec.realParmEst[3+survey][site]=pA;
		        SingSpec.realParmEst[3+survey+T][site]=pB;
				
                rA=getParam(3+survey+2*T,1,survey+2*T,site,survey,Params,0,SingSpec.LinkFn);
				SingSpec.realParmEst[3+survey+2*T][site]=rA;
                rB=getParam(3+survey+3*T,1,survey+3*T,site,survey,Params,0,SingSpec.LinkFn); 
				SingSpec.realParmEst[3+survey+3*T][site]=rB;
                rAB = rA*rB;
				switch(SingSpec.Alt_Param_Checked) {
                case 0:
                    delta=getParam(3+survey+4*T,1,survey+4*T,site,survey,Params,1,SingSpec.LinkFn);
					SingSpec.realParmEst[3+survey+4*T][site]=delta;
                    // check whether value is in allowable bounds
                    lower = (rA+rB-1.0)/(rA*rB); if (lower<0.0) lower=0.0;
                    upper = 1.0/rB; if (upper>(1.0/rA)) upper=1.0/rA;
                    if(delta < lower) return((fabs(delta-lower)+1.0)*1.0e10);
                    if(delta > upper) return((fabs(delta-upper)+1.0)*1.0e10);
                    rAB = rA*rB*delta;
					break;
                case 1:
                    rBA=rB; rBa=getParam(3+survey+4*T,1,survey+4*T,site,survey,Params,0,SingSpec.LinkFn);
					SingSpec.realParmEst[3+survey+4*T][site]=rBa;
                    rAB=rA*rBA; rB=rA*rBA+(1-rA)*rBa;
					break;
                case 2:
					rBa=getParam(3+survey+3*T,1,survey+3*T,site,0,Params,0,SingSpec.LinkFn);
					alf=50; if (rBa<1) alf=log(rBa/(1-rBa));
					rho=getParam(3+survey+4*T,1,survey+4*T,site,0,Params,0,expLnk);    //  logit(rBA) = alpha + gamma 
					gam=-50; if (rho>0) gam=log(rho);
					rBA=invlogit(alf+gam); 
					rB=rA*rBA+(1-rA)*rBa; 
					rAB=rA*rBA;  
					SingSpec.realParmEst[1][site]=rBa;
					SingSpec.realParmEst[2][site]=rho;
					break;
                case 3:                    //  newer parameterzation - rAb, rBa, odds-ratio(rAb:rA|B)
                    logitrAb=getParam(3+survey+2*T,1,survey+2*T,site,survey,Params,0,IDLnk);  //  untransformed = logit(rAb)
                    logitrBa=getParam(3+survey+3*T,1,survey+3*T,site,survey,Params,0,IDLnk);  //  untransformed = logit(rBa)
                    logtau=getParam(3+survey+4*T,1,survey+4*T,site,survey,Params,0,IDLnk);  
					tmp=exp(logitrAb+logitrBa+logtau); 
					denom=1+exp(logitrAb)+exp(logitrBa)+tmp;
					rAB=tmp/denom; 
			        rA=rAB/invlogit(logitrBa+logtau); rB=rAB/invlogit(logitrAb+logtau); 
					SingSpec.realParmEst[3+survey+2*T][site]=exp(logitrAb)/denom;
			        SingSpec.realParmEst[3+survey+3*T][site]=exp(logitrBa)/denom;
			        SingSpec.realParmEst[3+survey+4*T][site]=rAB;					
					break;
				case 4:                    //  newest parameterzation - alpha^A,alpha^B,alpha^AB
	                alfaA=getParam(3+survey+2*T,1,survey+2*T,site,survey,Params,0,IDLnk);
                    alfaB=getParam(3+survey+3*T,1,survey+3*T,site,survey,Params,0,IDLnk);                
                    alfaAB=getParam(3+survey+4*T,1,survey+4*T,site,survey,Params,0,IDLnk);                
                    rAb=exp(alfaA); raB=exp(alfaB); rAB=exp(alfaA+alfaB+alfaAB);
					K=1+rAb+raB+rAB; rAb/=K; raB/=K; rAB/=K;
					rA=rAb+rAB; rB=raB+rAB;
					SingSpec.realParmEst[3+survey+2*T][site]=rAb;
			        SingSpec.realParmEst[3+survey+3*T][site]=raB;
			        SingSpec.realParmEst[3+survey+4*T][site]=rAB;
					
					break;
                }
                //    rBa = Pr(detect B | not detected A)         case is important!
                //    rBA = Pr(detect B | detected A)
                //    rAB = Pr(detect A and B)
                //    rA = Pr(detect A)
                //    rB = Pr(detect B)
                switch (SingSpec.Data[site][survey]) {
                case 0:     // neither detected
                    likeAB *= (1.0-rA-rB+rAB); likeA *= (1.0-pA); likeB *= (1.0-pB); break;
                case 1: likeAB *= rA-rAB; likeA *= pA; seenA = 1; break;   // A detected, not B
                case 2: likeAB *= rB-rAB; likeB *= pB; seenB = 1; break;   // not A, B detected
                case 3: likeAB *= rAB;  seenA = seenB = 1;  break;         // A and B detected
				
				case 4: likeAB *= (1-rA); likeA *= (1-pA);  break;   // A not detected, B missing
				case 5: likeAB *= rA; likeA *= pA; seenA=1; break;             // A detected, B missing
				
				case 6: likeAB *= (1-rB); likeB *= (1-pB); break; // B not detected, A missing
				case 7: likeAB *= rB; likeB *= pB; seenB = 1;  break;        // B detected, A missing			
                }   // end switch
            } // end if missing
        } // end survey=0..T
        switch(1*seenA + 2*seenB) {
        case 0: tmp = likeAB + likeA + likeB + (1.0-psiA-psiB+psiAB); break;    //neither species detected  
        case 1: tmp = likeAB + likeA; break;    // A seen, not B
        case 2: tmp = likeAB + likeB; break;    // not A, B seen
        case 3: tmp = likeAB;  break;    // both seen
        } // end switch 
        if (tmp < SingSpec.nearzero) tmp = SingSpec.nearzero;
        sum-=SingSpec.det_hist_frq[site]*log(tmp);
    } // end individual site loop
    return(sum);
}
