# test2sp.R

test_that("model results", {

  setwd('c:/users/jhines/onedri~1/hines/a/aatmp'); library(RPresence)


  #cat('\n# ---- SINGLE-SEASON 2-SPECIES EXP VALUE MODEL ----\n')
  #
  x1=genpresEV2sp(N=100,K=4,
                  psi=c(.8,.3,.7),            # psiA, psiBA, psiBa
                  p=c(.6,.66,.7,.4,.5),       #  pA, pB, rA, rBA, rBa
                  gam=c(0, 0, 0),             #  gamA, gamBA, gamBa
                  eps=c(0, 0, 0),             #  epsA, epsBA, epsBa
                  SPS=4)                      #  surveys per season

  pao=createPao(data=x1$hst, frq=x1$frq)

  model=list(psi~SP+INT, p~SP+INT_o+INT_d)
  model=list(psi~ID, p~ID)

  EV2SPMOD2<-occMod(model,data=pao,type="so.2sp.1",outfile='tmp2.out',param='phi')

  EV2SPMOD<-occMod(model,data=pao,type="so.2sp.1",outfile='tmp1.out')
  print_one_site_estimates(EV2SPMOD)
  print_one_site_estimates(EV2SPMOD,derived=T)

  psiAB=EV2SPMOD$real$psi$psiA*EV2SPMOD$real$psi$psiBA
  psiB=psiAB+(1-EV2SPMOD$real$psi$psiA)*EV2SPMOD$real$psi$psiBa
  phi=psiAB/EV2SPMOD$real$psi$psiA/psiB

  EV2SPMOD2<-occMod(model,data=pao,type="so.2sp.1",outfile='tmp2.out',param='phi')
  print_one_site_estimates(EV2SPMOD2)
  print_one_site_estimates(EV2SPMOD2,derived=T)

  EV2SPMOD3<-occMod(model,data=pao,type="so.2sp.1",outfile='tmp3.out',param='nu')
  print_one_site_estimates(EV2SPMOD3)
  print_one_site_estimates(EV2SPMOD3,derived=T)

  EV2SPMOD4<-occMod(model,data=pao,type="so.2sp.1",outfile='tmp4.out',param='alfa')
  print_one_site_estimates(EV2SPMOD4)
  print_one_site_estimates(EV2SPMOD4,derived=T)

  #cat('# ---- TEST SINGLE-SEASON, TWO-SPECIES MODEL (DATA UNKNOWN) ----\n')
  #        Load a csv data file
  dethist<-read.csv(system.file("extdata/twosp_exmpl.csv",package="RPresence"),as.is=TRUE,header=FALSE)
  nsites=nrow(dethist); nsrvys=ncol(dethist)         #  set number of sites,surveys from det. history data
  dethist=matrix(as.integer(unlist(dethist)),nrow=nsites) # replace missing values (-) with NA
  cov1=cov2=NULL                                          # no covariates with these data

  ##          create input "pao" object, for use with occMod function
  data=createPao(dethist,unitcov=cov1,survcov=cov2,title="twosp example",save=T)

  ## fit some models
  #cat('---------------------- psi~SP, p~SP -------------------\n')
  mod1<-occMod(model=list(psi~SP,p~SP),data=data,type="so.2sp.1")
  #        For the 2-species models, RPresence automatically creates covariates
  #        for psi and p, named "SP".  Using this covariate for a parameter
  #        yields a different estimate for each of the two species.  So, the
  #        above model will give estimates of psiA, psiB, pA and pB (where
  #        A and B are the two species.

  #        Check out estimates of occupancy for each species...
  x=unique(mod1$real$psi$psiA)   # prob(occupancy for species A)
  #         est     se  lower  upper
  #unit1 0.7171 0.0668 0.5706 0.8287
  x=rbind(x,unique(mod1$real$psi$psiBA))     # prob(occupancy for species B when A is present)
  #         est     se  lower  upper
  #unit1 0.8403 0.0415 0.7415 0.9061
  x=rbind(x,unique(mod1$real$psi$psiBa))     # prob(occupancy for species B when A is absent
  #         est     se  lower  upper
  #unit1 0.8403 0.0415 0.7415 0.9061
  rownames(x)=c('psiA','psiBA','psiBa'); #print(x)


  #cat('---------------------- psi~SP+int, p~SP+INT_o -------------------\n')
  mod2<- occMod(model=list(psi~-1+ID, p~-1+ID), data=data, type="so.2sp.1")
  #        This model adds an interaction term to the occupancy estimate, meaning
  #         psiBA not = psiBa [occupancy of species B, given species A is present (psiBA)
  #           is not equal to occupancy of species B, given species A is absent (psiBa)]
  #        For detection, we've added an interaction term as well.  This term (INT_o) means
  #        detection depends on occupancy of the other species.  [pA not = rA, pB not = rB...
  #         detection of species A when only species A is present (pA) not equal to
  #         detection of species A when both species are present (rA).]
  #        To see all built-in covariates, type "help('occMod_2SP')".

  #        Check out estimates of occupancy for each species...
  #            Notice, estimates for species B differ depending on occupancy of species A
  #              (psiBA not = psiBa)
  x=unique(mod2$real$psi$psiA)   # prob(occupancy for species A)
  #         est     se  lower  upper
  #unit1 0.7298 0.0736 0.5652 0.8488
  x=rbind(x,unique(mod2$real$psi$psiBA))     # prob(occupancy for species B when A is present)
  #         est     se lower  upper
  #unit1 0.8756 0.0543 0.726 0.9492
  x=rbind(x,unique(mod2$real$psi$psiBa))     # prob(occupancy for species B when A is absent
  #         est     se  lower  upper
  #unit1 0.7521 0.1323 0.4301 0.9242
  rownames(x)=c('psiA','psiBA','psiBa');
  #print(x)

  #cat('---------------------- psi~SP+int, p~SP+INT_o -------------------\n')
  mod3<- occMod(model=list(psi~ID, p~ID), data=data, type="so.2sp.a")

  x=NULL; for (i in 1:length(mod3$derived$psi)) x=rbind(x,unique(mod3$derived$psi[[i]]))
  #print(x)

  mod3a<- occMod(model=list(psi~-1+ID, p~-1+ID), data=data, type="so.2sp.a",outfile='alfa_param.out')

  x=NULL; for (i in 1:length(mod3a$derived$psi)) x=rbind(x,unique(mod3a$derived$psi[[i]]))
  #print(x)

  #cat('two species AIC=',mod1$aic,'\n')

  #cat('# --- 2 SPECIES SINGLE SEASON FALSE POSITIVE MODEL... ----\n')

  y=read.table(system.file('extdata/twospfp_detdata.csv',package='RPresence'),header=FALSE,sep=',')
  w=read.table(system.file('extdata/twospfp_confdata.csv',package='RPresence'),header=FALSE,sep=',')
  pao=createPao(data=data.frame(y),survcov=data.frame(conf=as.numeric(unlist(w))),paoname='twospfp.pao')
  m2fp<-occMod(list(psi~SP+INT,p~SP+INT_o+INT_so,omeg~SP,conf~SP),type='so.2sp.fp',data=pao)


  expect_equal(EV2SPMOD$real$psi$psiA[1,1],.8)
  expect_equal(EV2SPMOD$real$psi$psiBA[1,1],.3)
  expect_equal(EV2SPMOD$real$psi$psiBa[1,1],.7)

  expect_equal(mod1$beta$psi[1,1],.930294)
  expect_equal(round(mod1$beta$psi[2,1],4),.7300)

  expect_equal(round(mod2$real$psi$psiA[1,1],4),.7454)

})
