# test_ss_ms.R
test_that("SINGLE-SEASON MULTI-STATE EXAMPLE (3 OCC STATES, PSI-R PARAM",{


  #    read detection history data from csv file...
  dethist<-read.csv(system.file("extdata/cal_owl_multistate_data.csv",
                                package="RPresence"),na.strings=".",colClasses="numeric")[,-1]
  nsites=nrow(dethist) #  set number of sites,surveys from det. history data
  nsrvys=ncol(dethist)
  frq=rep(1,nsites)
  #  create survey covariate to categorize surveys 1-2 as 1 period
  #                                    and surveys 3-5 as another period
  #    Since it's a survey covariate, it will be a NxT matrix (N=nsites, T=nsurveys)
  #    The covariate matrix would be:
  #         1 2 3 4 5
  #  site1 [1 1 2 2 2]
  #  site2 [1 1 2 2 2]
  #   :    [: : : : :]
  #  siteN [1 1 2 2 2]
  #    Filling in by cols means we repeat "1" N times, then "1" N times,
  #                  then "2" N times, then "2" N times, then "2" N times.
  #    We save it as a data frame (without the matrix dimensions).
  cov2=data.frame(PER=as.factor(c(rep(1,2*nsites),rep(2,3*nsites))))

  #          create input "pao" object, for use with occMod function
  data=createPao(data=dethist,frq=frq,survcov=cov2,title="Cal Owl example")
  suppressWarnings(xmods<-occMod(model=list(psi~1,R~1,p~STATE, delta~PER),
                                 data=data,type="do.ms.2"))

  #print_one_site_estimates(xmods)

  x=rbind(unique(xmods$real$psi[,1:2]),unique(xmods$real$R[,1:2]),
          unique(xmods$real$p[,1:2]),unique(xmods$real$delta[,1:2]))
  x$est=round(x$est,2); rownames(x)=c('psi','R','p1','p2','dlta1','dlta2')
  xmods_chk=round(x,3); xpsi=round(as.numeric(xmods$real$psi[1,1]),2)

  # --- SINGLE-SEASON MULTI-STATE EX (3 OCC STATES, PSI-R PARAM) (PG 231 IN BOOK)

  # --- EXP. VALUE DATA - MULTI-STATE, PSI/PHI PARAMETERIZATION... ----

  # generate expected value data for multi-season model (phi parameterization)
  K=6; nstates=3; p=matrix(0,nstates,nstates); p[2,2]<-p[3,2]<-p[3,3]<-.25; psi1=c(.167,.5,.333)
  phi=rbind(c(.5,.333),c(.5,.333),c(.5,.333)); phi=cbind(1-rowSums(phi),phi)

  x4=genpresEVms(N=100,K=K,nstates=3,psi1=psi1,phi=phi,p=p,SPS=2)  # generate exp. value data using RPresence function

  data=createPao(x4$hst,frq=x4$frq,nsurveyseason=rep(2,K/2))
  fixed=data.frame(param=paste0('p21(',1:K,')'),value=rep(0,K))

  ymods<-occMod(model=list(psi~STATE,phi~FROM_TO,p~OTSTATE),
                data=data,fixed=fixed,type="do.ms.1",threads=1)

  expect_equal(round(as.numeric(xmods_chk[1,1]),2),0.98)
  expect_equal(round(as.numeric(xmods_chk[2,1]),2),0.44)
  expect_equal(round(as.numeric(xmods_chk[3,1]),2),0.73)
  expect_equal(round(as.numeric(xmods_chk[4,1]),2),0.76)
  expect_equal(round(as.numeric(xmods_chk[5,1]),2),0.00)
  expect_equal(round(as.numeric(xmods_chk[6,1]),2),0.87)

  # ---- EXP. VALUE DATA - MULTI-STATE, PSI/PHI PARAMETERIZATION... ----
  expect_equal(round(unique(ymods$real$psi)[1,1],2),0.50)
  expect_equal(round(unique(ymods$real$psi)[2,1],2),0.33)
  expect_equal(round(unique(ymods$real$phi)[1,1],2),0.50)
  expect_equal(round(unique(ymods$real$phi)[2,1],2),0.33)
  expect_equal(round(unique(ymods$real$p)[1,1],2),0.25)
})
